require "spec_helper"

describe CliMetricCollector do

  let(:name) { 'name' }
  let(:extra) do
    { :smth => 1 }
  end
  let(:endpoint) { "http://wat.com" }

  subject do
    CliMetricCollector.new(name, extra)
  end

  before do
    CliMetricCollector.endpoint = endpoint
  end

  describe ".capture" do

    it "creates an instance and sendouts the capture" do
      collector = mock
      CliMetricCollector.
        should_receive(:new).
        with(name, extra).
        and_return(collector)
      collector.should_receive(:sendout)
      CliMetricCollector.capture name, extra

    end

  end

  describe "#sendout" do

    context "endpoint not configured" do
      let(:endpoint) { nil }

      it "informs the caller" do
        subject.should_receive(:puts) do |msg|
          msg.should == "CliMetricCollector is missing the endpoint, nothing sent" 
        end
        subject.sendout
      end
    end

    context "endpoint not configured" do
      let(:pid) { mock }

      before do
        subject.should_receive(:fork).and_return(pid)
      end

      context "in main process" do

        it "detaches the process" do
          Process.should_receive(:detach).with(pid)
          subject.sendout
        end

      end

      context "in subprocess" do
        let(:pid) { nil }

        it "executes curl" do
          subject.should_receive(:exec) do |command|
            data = '{"name":"name","data":{"smth":1}}'
            command.should == "curl -XPOST -H 'Content-Type: application/json' -d '#{data}' http://wat.com > /dev/null 2>&1"
          end
          subject.sendout
        end

      end

    end


  end

end

